# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine User Equipment API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *



from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class UserEquipment(object):
    """Identity Services Engine User Equipment API (version: 3.5.0).

    Wraps the Identity Services Engine User Equipment
    API and exposes the API as native Python
    methods that return native Python objects.


    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new UserEquipment
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(UserEquipment, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_user_equipments(self,
                            filter=None,
                            filter_type=None,
                            page=None,
                            size=None,
                            sortasc=None,
                            sortby=None,
                            headers=None,
                            **query_parameters):
        """Get user equipments.

        Args:
            page(int): page query parameter. Page number of the
                results to fetch. First page is 1.
            size(int): size query parameter. Number of results per
                page.
            filter(str): filter query parameter. Filter criteria for
                the query.  Format:
                attribute.operator.value Operators: EQ
                (equals), NEQ (not equals), GT (greater
                than), LT (less than),  STARTSW (starts
                with), NSTARTSW (not starts with), ENDSW
                (ends with),  NENDSW (not ends with),
                CONTAINS (contains), NCONTAINS (not
                contains) .
            filter_type(str): filterType query parameter. The
                logical operator for multiple filters
                (AND, OR).
            sortasc(bool): sortasc query parameter. Sort order (true
                for ascending, false for descending).
            sortby(str): sortby query parameter. Field name to sort
                by.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(page, (int, str, list))
        check_type(size, (int, str, list))
        check_type(filter, str)
        check_type(filter_type, str)
        check_type(sortasc, bool)
        check_type(sortby, str)

        _params = {
            'page':
                page,
            'size':
                size,
            'filter':
                filter,
            'filterType':
                filter_type,
            'sortasc':
                sortasc,
            'sortby':
                sortby,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/fiveg/user-equipment')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_c778156b01e7574bbcdd2b32704e6519_v3_5_0', _api_response)

    def get_user_equipments_generator(self,
                                      filter=None,
                                      filter_type=None,
                                      page=None,
                                      size=None,
                                      sortasc=None,
                                      sortby=None,
                                      headers=None,
                                      **query_parameters):
        """Get user equipments.

        Args:
            page(int): page query parameter. Page number of the
                results to fetch. First page is 1.
            size(int): size query parameter. Number of results per
                page.
            filter(str): filter query parameter. Filter criteria for
                the query.  Format:
                attribute.operator.value Operators: EQ
                (equals), NEQ (not equals), GT (greater
                than), LT (less than),  STARTSW (starts
                with), NSTARTSW (not starts with), ENDSW
                (ends with),  NENDSW (not ends with),
                CONTAINS (contains), NCONTAINS (not
                contains) .
            filter_type(str): filterType query parameter. The
                logical operator for multiple filters
                (AND, OR).
            sortasc(bool): sortasc query parameter. Sort order (true
                for ascending, false for descending).
            sortby(str): sortby query parameter. Field name to sort
                by.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            Generator: A generator object containing the following object.

              + RestResponse: REST response with following properties:

                  - headers(MyDict): response headers.
                  - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                    or the bracket notation.
                  - content(bytes): representation of the request's response
                  - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """

        yield from get_next_page(
            self.get_user_equipments, dict(
                filter=filter,
                filter_type=filter_type,
                page=page,
                size=size,
                sortasc=sortasc,
                sortby=sortby,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["nextPage", "href"],
            access_resource_list=["response"])

    def create_user_equipment(self,
                              description=None,
                              device_group=None,
                              imei=None,
                              headers=None,
                              payload=None,
                              active_validation=True,
                              **query_parameters):
        """Create a user equipment.

        Args:
            description(string): Description for User Equipment,
                property of the request body.
            device_group(string): Device or Endpoint Group, property
                of the request body.
            imei(string): IMEI for User Equipment, property of the
                request body. Constraints: maxLength set
                to 15 and minLength set to 14.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'description':
                    description,
                'deviceGroup':
                    device_group,
                'imei':
                    imei,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_a438c5c8e8e85ed3992b4b207c1e880d_v3_5_0')\
                .validate(_payload)

        e_url = ('/v1/fiveg/user-equipment')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_a438c5c8e8e85ed3992b4b207c1e880d_v3_5_0', _api_response)

    def get_user_equipment_by_id(self,
                                 user_equipment_id,
                                 headers=None,
                                 **query_parameters):
        """Get the user equipment for a given ID.

        Args:
            user_equipment_id(str): userEquipmentId path parameter.
                Unique ID for a user equipment object.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(user_equipment_id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'userEquipmentId': user_equipment_id,
        }

        e_url = ('/v1/fiveg/user-equipment/{userEquipmentId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_eb7751bd605264977bcce5b14fb889_v3_5_0', _api_response)

    def update_user_equipment(self,
                              user_equipment_id,
                              description=None,
                              device_group=None,
                              headers=None,
                              payload=None,
                              active_validation=True,
                              **query_parameters):
        """Update the user equipment for a given ID and request payload.

        Args:
            description(string): Description for User Equipment,
                property of the request body.
            device_group(string): Device or Endpoint Group, property
                of the request body.
            user_equipment_id(str): userEquipmentId path parameter.
                Unique ID for a user equipment object.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(user_equipment_id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'userEquipmentId': user_equipment_id,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'description':
                    description,
                'deviceGroup':
                    device_group,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_f7dd56c4a513c811cb9d8dbe79e92_v3_5_0')\
                .validate(_payload)

        e_url = ('/v1/fiveg/user-equipment/{userEquipmentId}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_f7dd56c4a513c811cb9d8dbe79e92_v3_5_0', _api_response)

    def delete_user_equipment(self,
                              user_equipment_id,
                              headers=None,
                              **query_parameters):
        """Delete the user equipment for a given ID.

        Args:
            user_equipment_id(str): userEquipmentId path parameter.
                Unique ID for a user equipment object.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(user_equipment_id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'userEquipmentId': user_equipment_id,
        }

        e_url = ('/v1/fiveg/user-equipment/{userEquipmentId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a42f015bf5e53679ed0802ce77d5212_v3_5_0', _api_response)

    def get_user_equipments_by_subscriber_id(self,
                                             subscriber_id,
                                             headers=None,
                                             **query_parameters):
        """Get user equipments associated with a subscriber GUID.

        Args:
            subscriber_id(str): subscriberId path parameter. Unique
                ID for a subscriber object.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(subscriber_id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'subscriberId': subscriber_id,
        }

        e_url = ('/v1/fiveg/user-equipment/subscriber/{subscriberId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b9bca0058945bf9b9346268536419bc_v3_5_0', _api_response)

    def get_user_equipment_by_i_m_e_i(self,
                                      imei,
                                      headers=None,
                                      **query_parameters):
        """Get a user equipment based on the IMEI.

        Args:
            imei(str): imei path parameter. IMEI for the user
                equipment object.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(imei, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'imei': imei,
        }

        e_url = ('/v1/fiveg/user-equipment/imei/{imei}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_e4e4412c567850adae980b9b299dc38b_v3_5_0', _api_response)

    def create_user_equipments_from_c_s_v(self,
                                          headers=None,
                                          **query_parameters):
        """Create user equipments from a CSV file.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/fiveg/user-equipment/csv')
        endpoint_full_url = apply_path_params(e_url, path_params)

        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params)

        return self._object_factory('bpm_c9c4b28095037a548ab2c5302903f_v3_5_0', _api_response)

    def bulk_user_equipment_operation(self,
                                      item_list=None,
                                      operation=None,
                                      headers=None,
                                      payload=None,
                                      active_validation=True,
                                      **query_parameters):
        """Create, update and delete multiple user equipments.

        Args:
            item_list(list): ItemList, property of the request body
                (list of objects).
            operation(): operation, property of the request body.
                Available values are 'Create', 'Update'
                and 'Delete'.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'operation':
                    operation,
                'ItemList':
                    item_list,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_e22bd55278af818b3e325fd95_v3_5_0')\
                .validate(_payload)

        e_url = ('/v1/fiveg/user-equipment/bulk')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_e22bd55278af818b3e325fd95_v3_5_0', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine internaluser API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *



from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class Internaluser(object):
    """Identity Services Engine internaluser API (version: 3.5.0).

    Wraps the Identity Services Engine internaluser
    API and exposes the API as native Python
    methods that return native Python objects.


    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new Internaluser
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(Internaluser, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_internaluser(self,
                         filter=None,
                         page=None,
                         size=None,
                         sortasc=None,
                         sortdsc=None,
                         headers=None,
                         **query_parameters):
        """Get-All.

        Args:
            filter(str): filter query parameter. Filter Supported
                Fields:[firstName, lastName,
                identityGroup, name, description, email,
                enabled].
            page(int): page query parameter. Page Number (0...N).
            size(int): size query parameter. Items by Page.
            sortdsc(str): sortdsc query parameter. Sorting Supported
                Fields:[daysTillPasswdExpiry,
                dateCreated, name, description,
                dateEnabled, dateModified].
            sortasc(str): sortasc query parameter. Sorting Supported
                Fields:[daysTillPasswdExpiry,
                dateCreated, name, description,
                dateEnabled, dateModified].
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(filter, str)
        check_type(page, (int, str, list))
        check_type(size, (int, str, list))
        check_type(sortdsc, str)
        check_type(sortasc, str)

        _params = {
            'filter':
                filter,
            'page':
                page,
            'size':
                size,
            'sortdsc':
                sortdsc,
            'sortasc':
                sortasc,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/internaluser/')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_c2892b453a29e791ef44472725b_v3_5_0', _api_response)

    def get_internaluser_generator(self,
                                   filter=None,
                                   page=None,
                                   size=None,
                                   sortasc=None,
                                   sortdsc=None,
                                   headers=None,
                                   **query_parameters):
        """Get-All.

        Args:
            filter(str): filter query parameter. Filter Supported
                Fields:[firstName, lastName,
                identityGroup, name, description, email,
                enabled].
            page(int): page query parameter. Page Number (0...N).
            size(int): size query parameter. Items by Page.
            sortdsc(str): sortdsc query parameter. Sorting Supported
                Fields:[daysTillPasswdExpiry,
                dateCreated, name, description,
                dateEnabled, dateModified].
            sortasc(str): sortasc query parameter. Sorting Supported
                Fields:[daysTillPasswdExpiry,
                dateCreated, name, description,
                dateEnabled, dateModified].
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            Generator: A generator object containing the following object.

              + RestResponse: REST response with following properties:

                  - headers(MyDict): response headers.
                  - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                    or the bracket notation.
                  - content(bytes): representation of the request's response
                  - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """

        yield from get_next_page(
            self.get_internaluser, dict(
                filter=filter,
                page=page,
                size=size,
                sortasc=sortasc,
                sortdsc=sortdsc,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["SearchResult", "nextPage", "href"],
            access_resource_list=["SearchResult", "resources"])

    def create_internaluser(self,
                            account_name_alias=None,
                            change_password=None,
                            custom_attributes=None,
                            date_created=None,
                            date_modified=None,
                            days_for_password_expiration=None,
                            description=None,
                            email=None,
                            enable_password=None,
                            enabled=None,
                            expiry_date=None,
                            expiry_date_enabled=None,
                            first_name=None,
                            id=None,
                            identity_groups=None,
                            last_name=None,
                            name=None,
                            password=None,
                            password_idstore=None,
                            password_never_expires=None,
                            headers=None,
                            payload=None,
                            active_validation=True,
                            **query_parameters):
        """Create.

        Args:
            account_name_alias(string): The Account Name Alias will
                be used to send email notifications
                about password expiration, property of
                the request body.
            change_password(boolean): changePassword, property of
                the request body.
            custom_attributes(string): Key value map, property of
                the request body.
            date_created(string): The date of the user creation,
                formatted as 'YYYY-MM-DD'. Read-only.,
                property of the request body.
            date_modified(string): The date of the last user
                modification by admin, formatted as
                'YYYY-MM-DD'. Read-only., property of
                the request body.
            days_for_password_expiration(number): This field is for
                read only, property of the request body.
            description(string): Description, property of the
                request body.
            email(string): email, property of the request body.
            enable_password(string): EnablePassword field is added
                in ISE 2.0 to support T+, property of
                the request body.
            enabled(boolean): Whether the user is enabled/disabled.
                To use it as filter, the values should
                be 'Enabled' or 'Disabled'. The values
                are case sensitive. For example, '[ERSOb
                jectURL]?filter=enabled.EQ.Enabled',
                property of the request body.
            expiry_date(string): to store the internal user's expiry
                date information. It's format is =
                'YYYY-MM-DD' , property of the request
                body.
            expiry_date_enabled(boolean): expiryDateEnabled,
                property of the request body.
            first_name(string): firstName, property of the request
                body.
            id(string): Id, property of the request body.
            identity_groups(string): CSV of identity group IDs.,
                property of the request body.
            last_name(string): lastName, property of the request
                body.
            name(string): name, property of the request body.
            password(string): password, property of the request
                body.
            password_idstore(string): the id store where the
                internal user's password is kept,
                property of the request body.
            password_never_expires(boolean): Set TRUE to indicate
                the user password never expires. This
                will not apply to Users who are also ISE
                Admins, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'changePassword':
                    change_password,
                'email':
                    email,
                'accountNameAlias':
                    account_name_alias,
                'enablePassword':
                    enable_password,
                'enabled':
                    enabled,
                'passwordNeverExpires':
                    password_never_expires,
                'daysForPasswordExpiration':
                    days_for_password_expiration,
                'customAttributes':
                    custom_attributes,
                'firstName':
                    first_name,
                'identityGroups':
                    identity_groups,
                'lastName':
                    last_name,
                'password':
                    password,
                'passwordIDStore':
                    password_idstore,
                'expiryDateEnabled':
                    expiry_date_enabled,
                'expiryDate':
                    expiry_date,
                'dateModified':
                    date_modified,
                'dateCreated':
                    date_created,
                'name':
                    name,
                'id':
                    id,
                'description':
                    description,
            }
            _payload = {
                'InternalUser': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_d70f5bcf7885af7b2df100e10b4c88e_v3_5_0')\
                .validate(_payload)

        e_url = ('/ers/config/internaluser/')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_d70f5bcf7885af7b2df100e10b4c88e_v3_5_0', _api_response)

    def get_internaluser_name_by_name(self,
                                      name,
                                      headers=None,
                                      **query_parameters):
        """Get-By-Name.

        Args:
            name(str): name path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'name': name,
        }

        e_url = ('/ers/config/internaluser/name/{name}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f403dda9440503191536993f569cc6f_v3_5_0', _api_response)

    def update_internaluser_name_by_name(self,
                                         name,
                                         account_name_alias=None,
                                         change_password=None,
                                         custom_attributes=None,
                                         date_created=None,
                                         date_modified=None,
                                         days_for_password_expiration=None,
                                         description=None,
                                         email=None,
                                         enable_password=None,
                                         enabled=None,
                                         expiry_date=None,
                                         expiry_date_enabled=None,
                                         first_name=None,
                                         id=None,
                                         identity_groups=None,
                                         last_name=None,
                                         password=None,
                                         password_idstore=None,
                                         password_never_expires=None,
                                         headers=None,
                                         payload=None,
                                         active_validation=True,
                                         **query_parameters):
        """Update-By-Name.

        Args:
            account_name_alias(string): The Account Name Alias will
                be used to send email notifications
                about password expiration, property of
                the request body.
            change_password(boolean): changePassword, property of
                the request body.
            custom_attributes(string): Key value map, property of
                the request body.
            date_created(string): The date of the user creation,
                formatted as 'YYYY-MM-DD'. Read-only.,
                property of the request body.
            date_modified(string): The date of the last user
                modification by admin, formatted as
                'YYYY-MM-DD'. Read-only., property of
                the request body.
            days_for_password_expiration(number): This field is for
                read only, property of the request body.
            description(string): Description, property of the
                request body.
            email(string): email, property of the request body.
            enable_password(string): EnablePassword field is added
                in ISE 2.0 to support T+, property of
                the request body.
            enabled(boolean): Whether the user is enabled/disabled.
                To use it as filter, the values should
                be 'Enabled' or 'Disabled'. The values
                are case sensitive. For example, '[ERSOb
                jectURL]?filter=enabled.EQ.Enabled',
                property of the request body.
            expiry_date(string): to store the internal user's expiry
                date information. It's format is =
                'YYYY-MM-DD' , property of the request
                body.
            expiry_date_enabled(boolean): expiryDateEnabled,
                property of the request body.
            first_name(string): firstName, property of the request
                body.
            id(string): Id, property of the request body.
            identity_groups(string): CSV of identity group IDs.,
                property of the request body.
            last_name(string): lastName, property of the request
                body.
            name(string): name, property of the request body.
            password(string): password, property of the request
                body.
            password_idstore(string): the id store where the
                internal user's password is kept,
                property of the request body.
            password_never_expires(boolean): Set TRUE to indicate
                the user password never expires. This
                will not apply to Users who are also ISE
                Admins, property of the request body.
            name(str): name path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'name': name,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'changePassword':
                    change_password,
                'email':
                    email,
                'accountNameAlias':
                    account_name_alias,
                'enablePassword':
                    enable_password,
                'enabled':
                    enabled,
                'passwordNeverExpires':
                    password_never_expires,
                'daysForPasswordExpiration':
                    days_for_password_expiration,
                'customAttributes':
                    custom_attributes,
                'firstName':
                    first_name,
                'identityGroups':
                    identity_groups,
                'lastName':
                    last_name,
                'password':
                    password,
                'passwordIDStore':
                    password_idstore,
                'expiryDateEnabled':
                    expiry_date_enabled,
                'expiryDate':
                    expiry_date,
                'dateModified':
                    date_modified,
                'dateCreated':
                    date_created,
                'name':
                    name,
                'id':
                    id,
                'description':
                    description,
            }
            _payload = {
                'InternalUser': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_19d9509db339e3b27dc56b37_v3_5_0')\
                .validate(_payload)

        e_url = ('/ers/config/internaluser/name/{name}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_19d9509db339e3b27dc56b37_v3_5_0', _api_response)

    def delete_internaluser_name_by_name(self,
                                         name,
                                         headers=None,
                                         **query_parameters):
        """Delete-By-Name.

        Args:
            name(str): name path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'name': name,
        }

        e_url = ('/ers/config/internaluser/name/{name}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b4e2fc3e595aa1be86d6589614b9_v3_5_0', _api_response)

    def patch_internaluser_name_by_name(self,
                                        name,
                                        account_name_alias=None,
                                        change_password=None,
                                        custom_attributes=None,
                                        date_created=None,
                                        date_modified=None,
                                        days_for_password_expiration=None,
                                        description=None,
                                        email=None,
                                        enable_password=None,
                                        enabled=None,
                                        expiry_date=None,
                                        expiry_date_enabled=None,
                                        first_name=None,
                                        id=None,
                                        identity_groups=None,
                                        last_name=None,
                                        password=None,
                                        password_idstore=None,
                                        password_never_expires=None,
                                        headers=None,
                                        payload=None,
                                        active_validation=True,
                                        **query_parameters):
        """Update any attribute subset. Only attributes that sent will be
        affected.

        Args:
            account_name_alias(string): The Account Name Alias will
                be used to send email notifications
                about password expiration, property of
                the request body.
            change_password(boolean): changePassword, property of
                the request body.
            custom_attributes(string): Key value map, property of
                the request body.
            date_created(string): The date of the user creation,
                formatted as 'YYYY-MM-DD'. Read-only.,
                property of the request body.
            date_modified(string): The date of the last user
                modification by admin, formatted as
                'YYYY-MM-DD'. Read-only., property of
                the request body.
            days_for_password_expiration(number): This field is for
                read only, property of the request body.
            description(string): Description, property of the
                request body.
            email(string): email, property of the request body.
            enable_password(string): EnablePassword field is added
                in ISE 2.0 to support T+, property of
                the request body.
            enabled(boolean): Whether the user is enabled/disabled.
                To use it as filter, the values should
                be 'Enabled' or 'Disabled'. The values
                are case sensitive. For example, '[ERSOb
                jectURL]?filter=enabled.EQ.Enabled',
                property of the request body.
            expiry_date(string): to store the internal user's expiry
                date information. It's format is =
                'YYYY-MM-DD' , property of the request
                body.
            expiry_date_enabled(boolean): expiryDateEnabled,
                property of the request body.
            first_name(string): firstName, property of the request
                body.
            id(string): Id, property of the request body.
            identity_groups(string): CSV of identity group IDs.,
                property of the request body.
            last_name(string): lastName, property of the request
                body.
            name(string): name, property of the request body.
            password(string): password, property of the request
                body.
            password_idstore(string): the id store where the
                internal user's password is kept,
                property of the request body.
            password_never_expires(boolean): Set TRUE to indicate
                the user password never expires. This
                will not apply to Users who are also ISE
                Admins, property of the request body.
            name(str): name path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'name': name,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'changePassword':
                    change_password,
                'email':
                    email,
                'accountNameAlias':
                    account_name_alias,
                'enablePassword':
                    enable_password,
                'enabled':
                    enabled,
                'passwordNeverExpires':
                    password_never_expires,
                'daysForPasswordExpiration':
                    days_for_password_expiration,
                'customAttributes':
                    custom_attributes,
                'firstName':
                    first_name,
                'identityGroups':
                    identity_groups,
                'lastName':
                    last_name,
                'password':
                    password,
                'passwordIDStore':
                    password_idstore,
                'expiryDateEnabled':
                    expiry_date_enabled,
                'expiryDate':
                    expiry_date,
                'dateModified':
                    date_modified,
                'dateCreated':
                    date_created,
                'name':
                    name,
                'id':
                    id,
                'description':
                    description,
            }
            _payload = {
                'InternalUser': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_b243462eb0589d866791af0b0d700d_v3_5_0')\
                .validate(_payload)

        e_url = ('/ers/config/internaluser/name/{name}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.patch(endpoint_full_url, params=_params,
                                                headers=_headers,
                                                **request_params)
        else:
            _api_response = self._session.patch(endpoint_full_url, params=_params,
                                                **request_params)

        return self._object_factory('bpm_b243462eb0589d866791af0b0d700d_v3_5_0', _api_response)

    def get_internaluser_by_id(self,
                               id,
                               headers=None,
                               **query_parameters):
        """Get-By-Id.

        Args:
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/internaluser/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_bacf1abfc35e509183c9a7f055cbbfec_v3_5_0', _api_response)

    def update_internaluser_by_id(self,
                                  id,
                                  account_name_alias=None,
                                  change_password=None,
                                  custom_attributes=None,
                                  date_created=None,
                                  date_modified=None,
                                  days_for_password_expiration=None,
                                  description=None,
                                  email=None,
                                  enable_password=None,
                                  enabled=None,
                                  expiry_date=None,
                                  expiry_date_enabled=None,
                                  first_name=None,
                                  identity_groups=None,
                                  last_name=None,
                                  name=None,
                                  password=None,
                                  password_idstore=None,
                                  password_never_expires=None,
                                  headers=None,
                                  payload=None,
                                  active_validation=True,
                                  **query_parameters):
        """Update.

        Args:
            account_name_alias(string): The Account Name Alias will
                be used to send email notifications
                about password expiration, property of
                the request body.
            change_password(boolean): changePassword, property of
                the request body.
            custom_attributes(string): Key value map, property of
                the request body.
            date_created(string): The date of the user creation,
                formatted as 'YYYY-MM-DD'. Read-only.,
                property of the request body.
            date_modified(string): The date of the last user
                modification by admin, formatted as
                'YYYY-MM-DD'. Read-only., property of
                the request body.
            days_for_password_expiration(number): This field is for
                read only, property of the request body.
            description(string): Description, property of the
                request body.
            email(string): email, property of the request body.
            enable_password(string): EnablePassword field is added
                in ISE 2.0 to support T+, property of
                the request body.
            enabled(boolean): Whether the user is enabled/disabled.
                To use it as filter, the values should
                be 'Enabled' or 'Disabled'. The values
                are case sensitive. For example, '[ERSOb
                jectURL]?filter=enabled.EQ.Enabled',
                property of the request body.
            expiry_date(string): to store the internal user's expiry
                date information. It's format is =
                'YYYY-MM-DD' , property of the request
                body.
            expiry_date_enabled(boolean): expiryDateEnabled,
                property of the request body.
            first_name(string): firstName, property of the request
                body.
            id(string): Id, property of the request body.
            identity_groups(string): CSV of identity group IDs.,
                property of the request body.
            last_name(string): lastName, property of the request
                body.
            name(string): name, property of the request body.
            password(string): password, property of the request
                body.
            password_idstore(string): the id store where the
                internal user's password is kept,
                property of the request body.
            password_never_expires(boolean): Set TRUE to indicate
                the user password never expires. This
                will not apply to Users who are also ISE
                Admins, property of the request body.
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'changePassword':
                    change_password,
                'email':
                    email,
                'accountNameAlias':
                    account_name_alias,
                'enablePassword':
                    enable_password,
                'enabled':
                    enabled,
                'passwordNeverExpires':
                    password_never_expires,
                'daysForPasswordExpiration':
                    days_for_password_expiration,
                'customAttributes':
                    custom_attributes,
                'firstName':
                    first_name,
                'identityGroups':
                    identity_groups,
                'lastName':
                    last_name,
                'password':
                    password,
                'passwordIDStore':
                    password_idstore,
                'expiryDateEnabled':
                    expiry_date_enabled,
                'expiryDate':
                    expiry_date,
                'dateModified':
                    date_modified,
                'dateCreated':
                    date_created,
                'name':
                    name,
                'id':
                    id,
                'description':
                    description,
            }
            _payload = {
                'InternalUser': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_f7227b280b745b94bb801369b168a529_v3_5_0')\
                .validate(_payload)

        e_url = ('/ers/config/internaluser/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_f7227b280b745b94bb801369b168a529_v3_5_0', _api_response)

    def delete_internaluser_by_id(self,
                                  id,
                                  headers=None,
                                  **query_parameters):
        """Delete.

        Args:
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/internaluser/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_dcf28db5184e51139b15f9ffccd10b67_v3_5_0', _api_response)

    def patch_internaluser_by_id(self,
                                 id,
                                 account_name_alias=None,
                                 change_password=None,
                                 custom_attributes=None,
                                 date_created=None,
                                 date_modified=None,
                                 days_for_password_expiration=None,
                                 description=None,
                                 email=None,
                                 enable_password=None,
                                 enabled=None,
                                 expiry_date=None,
                                 expiry_date_enabled=None,
                                 first_name=None,
                                 identity_groups=None,
                                 last_name=None,
                                 name=None,
                                 password=None,
                                 password_idstore=None,
                                 password_never_expires=None,
                                 headers=None,
                                 payload=None,
                                 active_validation=True,
                                 **query_parameters):
        """Update any attribute subset. Only attributes that sent will be
        affected.

        Args:
            account_name_alias(string): The Account Name Alias will
                be used to send email notifications
                about password expiration, property of
                the request body.
            change_password(boolean): changePassword, property of
                the request body.
            custom_attributes(string): Key value map, property of
                the request body.
            date_created(string): The date of the user creation,
                formatted as 'YYYY-MM-DD'. Read-only.,
                property of the request body.
            date_modified(string): The date of the last user
                modification by admin, formatted as
                'YYYY-MM-DD'. Read-only., property of
                the request body.
            days_for_password_expiration(number): This field is for
                read only, property of the request body.
            description(string): Description, property of the
                request body.
            email(string): email, property of the request body.
            enable_password(string): EnablePassword field is added
                in ISE 2.0 to support T+, property of
                the request body.
            enabled(boolean): Whether the user is enabled/disabled.
                To use it as filter, the values should
                be 'Enabled' or 'Disabled'. The values
                are case sensitive. For example, '[ERSOb
                jectURL]?filter=enabled.EQ.Enabled',
                property of the request body.
            expiry_date(string): to store the internal user's expiry
                date information. It's format is =
                'YYYY-MM-DD' , property of the request
                body.
            expiry_date_enabled(boolean): expiryDateEnabled,
                property of the request body.
            first_name(string): firstName, property of the request
                body.
            id(string): Id, property of the request body.
            identity_groups(string): CSV of identity group IDs.,
                property of the request body.
            last_name(string): lastName, property of the request
                body.
            name(string): name, property of the request body.
            password(string): password, property of the request
                body.
            password_idstore(string): the id store where the
                internal user's password is kept,
                property of the request body.
            password_never_expires(boolean): Set TRUE to indicate
                the user password never expires. This
                will not apply to Users who are also ISE
                Admins, property of the request body.
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'changePassword':
                    change_password,
                'email':
                    email,
                'accountNameAlias':
                    account_name_alias,
                'enablePassword':
                    enable_password,
                'enabled':
                    enabled,
                'passwordNeverExpires':
                    password_never_expires,
                'daysForPasswordExpiration':
                    days_for_password_expiration,
                'customAttributes':
                    custom_attributes,
                'firstName':
                    first_name,
                'identityGroups':
                    identity_groups,
                'lastName':
                    last_name,
                'password':
                    password,
                'passwordIDStore':
                    password_idstore,
                'expiryDateEnabled':
                    expiry_date_enabled,
                'expiryDate':
                    expiry_date,
                'dateModified':
                    date_modified,
                'dateCreated':
                    date_created,
                'name':
                    name,
                'id':
                    id,
                'description':
                    description,
            }
            _payload = {
                'InternalUser': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_cd1bba62cec58edb1b2511ec2b15f00_v3_5_0')\
                .validate(_payload)

        e_url = ('/ers/config/internaluser/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.patch(endpoint_full_url, params=_params,
                                                headers=_headers,
                                                **request_params)
        else:
            _api_response = self._session.patch(endpoint_full_url, params=_params,
                                                **request_params)

        return self._object_factory('bpm_cd1bba62cec58edb1b2511ec2b15f00_v3_5_0', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine sgacl API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *



from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class Sgacl(object):
    """Identity Services Engine sgacl API (version: 3.5.0).

    Wraps the Identity Services Engine sgacl
    API and exposes the API as native Python
    methods that return native Python objects.


    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new Sgacl
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(Sgacl, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_sgacl(self,
                  filter=None,
                  page=None,
                  size=None,
                  sortasc=None,
                  sortdsc=None,
                  headers=None,
                  **query_parameters):
        """Get-All.

        Args:
            filter(str): filter query parameter. Filter Supported
                Fields:[ipVersion, sgAclType, name,
                description].
            page(int): page query parameter. Page Number (0...N).
            size(int): size query parameter. Items by Page.
            sortdsc(str): sortdsc query parameter. Sorting Supported
                Fields:[ipVersion, sgAclType, name,
                description].
            sortasc(str): sortasc query parameter. Sorting Supported
                Fields:[ipVersion, sgAclType, name,
                description].
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(filter, str)
        check_type(page, (int, str, list))
        check_type(size, (int, str, list))
        check_type(sortdsc, str)
        check_type(sortasc, str)

        _params = {
            'filter':
                filter,
            'page':
                page,
            'size':
                size,
            'sortdsc':
                sortdsc,
            'sortasc':
                sortasc,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/sgacl/')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f6e4b32a9cf558098b96224d778f88dd_v3_5_0', _api_response)

    def get_sgacl_generator(self,
                            filter=None,
                            page=None,
                            size=None,
                            sortasc=None,
                            sortdsc=None,
                            headers=None,
                            **query_parameters):
        """Get-All.

        Args:
            filter(str): filter query parameter. Filter Supported
                Fields:[ipVersion, sgAclType, name,
                description].
            page(int): page query parameter. Page Number (0...N).
            size(int): size query parameter. Items by Page.
            sortdsc(str): sortdsc query parameter. Sorting Supported
                Fields:[ipVersion, sgAclType, name,
                description].
            sortasc(str): sortasc query parameter. Sorting Supported
                Fields:[ipVersion, sgAclType, name,
                description].
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            Generator: A generator object containing the following object.

              + RestResponse: REST response with following properties:

                  - headers(MyDict): response headers.
                  - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                    or the bracket notation.
                  - content(bytes): representation of the request's response
                  - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """

        yield from get_next_page(
            self.get_sgacl, dict(
                filter=filter,
                page=page,
                size=size,
                sortasc=sortasc,
                sortdsc=sortdsc,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["SearchResult", "nextPage", "href"],
            access_resource_list=["SearchResult", "resources"])

    def create_sgacl(self,
                     aclcontent=None,
                     description=None,
                     generation_id=None,
                     id=None,
                     ip_version=None,
                     modelled_content=None,
                     name=None,
                     read_only=None,
                     sg_acl_type=None,
                     headers=None,
                     payload=None,
                     active_validation=True,
                     **query_parameters):
        """Create.

        Args:
            aclcontent(string): aclcontent, property of the request
                body.
            description(string): Description, property of the
                request body.
            generation_id(string): GenerationId is a read only
                attribute and is being generated by the
                server., property of the request body.
            id(string): Id, property of the request body.
            ip_version(string): Allowed
                values:IPV4,IPV6,IP_AGNOSTIC, property
                of the request body.
            modelled_content(object): Modelled content of contract,
                property of the request body.
            name(string): name, property of the request body.
            read_only(boolean): readOnly, property of the request
                body.
            sg_acl_type(string): SGACL type  Trustsec or Traffic
                Steering. To use it as filter, the
                values should be 'TRAFFIC_STEERING' or
                'TRUSTSEC'. The values are case
                sensitive. For example, '[ERSObjectURL]?
                filter=sgAclType.eq.TRUSTSEC' or '[ERSOb
                jectURL]?filter=sgAclType.eq.TRAFFIC_STE
                ERING', property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'aclcontent':
                    aclcontent,
                'ipVersion':
                    ip_version,
                'generationId':
                    generation_id,
                'readOnly':
                    read_only,
                'sgAclType':
                    sg_acl_type,
                'modelledContent':
                    modelled_content,
                'name':
                    name,
                'id':
                    id,
                'description':
                    description,
            }
            _payload = {
                'Sgacl': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_cdf3223c24850c993108eb55d159ae5_v3_5_0')\
                .validate(_payload)

        e_url = ('/ers/config/sgacl/')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_cdf3223c24850c993108eb55d159ae5_v3_5_0', _api_response)

    def get_sgacl_by_id(self,
                        id,
                        headers=None,
                        **query_parameters):
        """Get-By-Id.

        Args:
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/sgacl/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a50d1bd34d5f593aadf8eb02083c67b0_v3_5_0', _api_response)

    def update_sgacl_by_id(self,
                           id,
                           aclcontent=None,
                           description=None,
                           generation_id=None,
                           ip_version=None,
                           modelled_content=None,
                           name=None,
                           read_only=None,
                           sg_acl_type=None,
                           headers=None,
                           payload=None,
                           active_validation=True,
                           **query_parameters):
        """Update.

        Args:
            aclcontent(string): aclcontent, property of the request
                body.
            description(string): Description, property of the
                request body.
            generation_id(string): GenerationId is a read only
                attribute and is being generated by the
                server., property of the request body.
            id(string): Id, property of the request body.
            ip_version(string): Allowed
                values:IPV4,IPV6,IP_AGNOSTIC, property
                of the request body.
            modelled_content(object): Modelled content of contract,
                property of the request body.
            name(string): name, property of the request body.
            read_only(boolean): readOnly, property of the request
                body.
            sg_acl_type(string): SGACL type  Trustsec or Traffic
                Steering. To use it as filter, the
                values should be 'TRAFFIC_STEERING' or
                'TRUSTSEC'. The values are case
                sensitive. For example, '[ERSObjectURL]?
                filter=sgAclType.eq.TRUSTSEC' or '[ERSOb
                jectURL]?filter=sgAclType.eq.TRAFFIC_STE
                ERING', property of the request body.
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'aclcontent':
                    aclcontent,
                'ipVersion':
                    ip_version,
                'generationId':
                    generation_id,
                'readOnly':
                    read_only,
                'sgAclType':
                    sg_acl_type,
                'modelledContent':
                    modelled_content,
                'name':
                    name,
                'id':
                    id,
                'description':
                    description,
            }
            _payload = {
                'Sgacl': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_afc81cd1e25c50319f75606b97c23b3d_v3_5_0')\
                .validate(_payload)

        e_url = ('/ers/config/sgacl/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_afc81cd1e25c50319f75606b97c23b3d_v3_5_0', _api_response)

    def delete_sgacl_by_id(self,
                           id,
                           headers=None,
                           **query_parameters):
        """Delete.

        Args:
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/sgacl/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b0a2bea8bfec52b68663ef3f7ac6d7a7_v3_5_0', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine NativeSupplicantProfile API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from builtins import *
from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class NativeSupplicantProfile(object):
    """Identity Services Engine NativeSupplicantProfile API (version: 3.2_beta).

    Wraps the Identity Services Engine NativeSupplicantProfile
    API and exposes the API as native Python
    methods that return native Python objects.

    | Native supplicant profile API provides the ability to update, delete and search native supplicant profiles.

    **Revision History**

    +----------------+----------------------+-----------------------+---------------------------+
    | **Revision #** | **Resource Version** | **Cisco ISE Version** | **Description**           |
    +----------------+----------------------+-----------------------+---------------------------+
    | 0              | 1.0                  | 2.2                   | Initial Cisco ISE Version |
    +----------------+----------------------+-----------------------+---------------------------+

    |

    **Resource Definition**

    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+
    | **Attribute**           | **Type**    | **Required** | **Description**                                                                | **Example Values**                                                                                 |
    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+
    | name                    | String      | Yes          | Resource Name                                                                  | Cisco-ISE-NSP                                                                                      |
    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+
    | id                      | String      | No           | Resource UUID, mandatory for update                                            | b8ce01e6-b150-4d4e-9698-40e48d5e0197                                                               |
    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+
    | description             | String      | No           |                                                                                | Pre-configured Native Supplicant Profile. The SSID Will Need To Be Customized For Your Environment |
    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+
    | wirelessProfiles        | List        | Yes          | List of Wireless profiles                                                      |                                                                                                    |
    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+
    | - ssid                  | String      | Yes          | SSID for the wireless profile                                                  | ISE                                                                                                |
    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+
    | - allowedProtocol       | Enum        | Yes          | Allowed protocol for the wireless profile.                                     | TLS                                                                                                |
    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+
    | - certificateTemplateId | String      | No           | Certificate template ID                                                        | 0ca8f1b6-500d-560b-e053-75189a0ab0d1                                                               |
    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+
    | - actionType            | Enum        | Yes          | Action type for WifiProfile.                                                   | UPDATE                                                                                             |
    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+
    | - previousSsid          | String      | Yes          | Previous ssid for WifiProfile (required for updating existing WirelessProfile) | ssid1                                                                                              |
    +-------------------------+-------------+--------------+--------------------------------------------------------------------------------+----------------------------------------------------------------------------------------------------+

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new NativeSupplicantProfile
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(NativeSupplicantProfile, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_native_supplicant_profile_by_id(self,
                                            id,
                                            headers=None,
                                            **query_parameters):
        """This API allows the client to get a native supplicant profile by
        ID.

        Args:
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           str)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/nspprofile/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d1b9755414c5dcbb61987b2dd06839a_v3_2_beta', _api_response)

    def get_by_id(self,
                  id,
                  headers=None,
                  **query_parameters):
        """Alias for `get_native_supplicant_profile_by_id <#ciscoisesdk.
        api.v3_2_beta.native_supplicant_profile.
        NativeSupplicantProfile.get_native_supplicant_profile_by_id>`_
        """
        return self.get_native_supplicant_profile_by_id(
            id=id,
            headers=headers,
            **query_parameters
        )

    def update_native_supplicant_profile_by_id(self,
                                               id,
                                               description=None,
                                               name=None,
                                               wireless_profiles=None,
                                               headers=None,
                                               payload=None,
                                               active_validation=True,
                                               **query_parameters):
        """This API allows the client to update a native supplicant
        profile.

        Args:
            description(string): description, property of the
                request body.
            id(string): id, property of the request body.
            name(string): name, property of the request body.
            wireless_profiles(list): wirelessProfiles, property of
                the request body (list of objects).
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           str)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'id':
                    id,
                'name':
                    name,
                'description':
                    description,
                'wirelessProfiles':
                    wireless_profiles,
            }
            _payload = {
                'ERSNSPProfile': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_c54a2ad63f46527dbec140a05f1213b7_v3_2_beta')\
                .validate(_payload)

        e_url = ('/ers/config/nspprofile/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_c54a2ad63f46527dbec140a05f1213b7_v3_2_beta', _api_response)

    def update_by_id(self,
                     id,
                     description=None,
                     name=None,
                     wireless_profiles=None,
                     headers=None,
                     payload=None,
                     active_validation=True,
                     **query_parameters):
        """Alias for `update_native_supplicant_profile_by_id <#ciscoisesdk.
        api.v3_2_beta.native_supplicant_profile.
        NativeSupplicantProfile.update_native_supplicant_profile_by_id>`_
        """
        return self.update_native_supplicant_profile_by_id(
            id=id,
            description=description,
            name=name,
            wireless_profiles=wireless_profiles,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def delete_native_supplicant_profile_by_id(self,
                                               id,
                                               headers=None,
                                               **query_parameters):
        """This API deletes a native supplicant profile.

        Args:
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           str)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/nspprofile/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fff9d421c78597d98a54dd08a9a99f9_v3_2_beta', _api_response)

    def delete_by_id(self,
                     id,
                     headers=None,
                     **query_parameters):
        """Alias for `delete_native_supplicant_profile_by_id <#ciscoisesdk.
        api.v3_2_beta.native_supplicant_profile.
        NativeSupplicantProfile.delete_native_supplicant_profile_by_id>`_
        """
        return self.delete_native_supplicant_profile_by_id(
            id=id,
            headers=headers,
            **query_parameters
        )

    def get_native_supplicant_profile(self,
                                      page=None,
                                      size=None,
                                      headers=None,
                                      **query_parameters):
        """This API allows the client to get all the native supplicant
        profiles.

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           str)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(page, (int, str, list))
        check_type(size, (int, str, list))

        _params = {
            'page':
                page,
            'size':
                size,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/nspprofile')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fa9802505d7bbdf85b951581db47_v3_2_beta', _api_response)

    def get_all(self,
                page=None,
                size=None,
                headers=None,
                **query_parameters):
        """Alias for `get_native_supplicant_profile <#ciscoisesdk.
        api.v3_2_beta.native_supplicant_profile.
        NativeSupplicantProfile.get_native_supplicant_profile>`_
        """
        return self.get_native_supplicant_profile(
            page=page,
            size=size,
            headers=headers,
            **query_parameters
        )

    def get_native_supplicant_profile_generator(self,
                                                page=None,
                                                size=None,
                                                headers=None,
                                                **query_parameters):
        """This API allows the client to get all the native supplicant
        profiles.

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            Generator: A generator object containing the following object.

              + RestResponse: REST response with following properties:

                  - headers(MyDict): response headers.
                  - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                    or the bracket notation.
                  - content(bytes): representation of the request's response
                  - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """

        yield from get_next_page(
            self.get_native_supplicant_profile, dict(
                page=page,
                size=size,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["SearchResult", "nextPage", "href"],
            access_resource_list=["SearchResult", "resources"])

    def get_all_generator(self,
                          page=None,
                          size=None,
                          headers=None,
                          **query_parameters):
        """Alias for `get_native_supplicant_profile_generator <#ciscoisesdk.
        api.v3_2_beta.native_supplicant_profile.
        NativeSupplicantProfile.get_native_supplicant_profile_generator>`_
        """
        yield from get_next_page(
            self.get_native_supplicant_profile, dict(
                page=page,
                size=size,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["SearchResult", "nextPage", "href"],
            access_resource_list=["SearchResult", "resources"])

    def get_version(self,
                    headers=None,
                    **query_parameters):
        """This API helps to retrieve the version information related to
        the native supplicant profile.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/nspprofile/versioninfo')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f577c55d36b05178b0275dd88c71e118_v3_2_beta', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine Patch API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *



from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class Patch(object):
    """Identity Services Engine Patch API (version: 3.5.0).

    Wraps the Identity Services Engine Patch
    API and exposes the API as native Python
    methods that return native Python objects.


    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new Patch
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(Patch, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def run_patch_pre_checks(self,
                             hostnames=None,
                             patch_bundle_name=None,
                             pre_check_report_id=None,
                             pre_checks=None,
                             re_trigger=None,
                             repo_name=None,
                             upgrade_type=None,
                             headers=None,
                             payload=None,
                             active_validation=True,
                             **query_parameters):
        """Initiates prechecks execution on PPAN for complete deployment.
        It returns a precheck report id, which can be used to
        trigger patch installation later. Patch installation can
        happen in three modes :    Full Patch Installation ->
        Hostname needs to be an empty array. First patch will
        get installed on PPAN and then it will get installed on
        all other nodes parallelly.   Sequential Patch
        Installation -> Hostname needs to be an empty array.
        First patch will get installed on PPAN and then it will
        get installed on all other nodes sequentially.   Split
        -> This will happen in batches / iterations. Host name
        is mandatory. In first iteration only PPAN should be
        sent as part of hostname. Once the PPAN installation is
        successfull then remaining nodes can be sent in batches.
        upgradeType param can be specified as FULL_PATCH for
        full patch installation, SPLIT_PATCH for split patch
        installation and SEQUENTIAL_PATCH for sequential patch
        installation. upgrade type, patch bundle name,
        repository name are mandatory parameters. .

        Args:
            hostnames(list): hostnames, property of the request body
                (list of strings).
            patch_bundle_name(string): patchBundleName, property of
                the request body.
            pre_check_report_id(string): preCheckReportID, property
                of the request body.
            pre_checks(list): Array of prechecks that needs to be
                executed., property of the request body
                (list of strings).
            re_trigger(boolean): reTrigger, property of the request
                body.
            repo_name(string): repoName, property of the request
                body.
            upgrade_type(string): upgradeType, property of the
                request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'preChecks':
                    pre_checks,
                'hostnames':
                    hostnames,
                'upgradeType':
                    upgrade_type,
                'repoName':
                    repo_name,
                'patchBundleName':
                    patch_bundle_name,
                'preCheckReportID':
                    pre_check_report_id,
                'reTrigger':
                    re_trigger,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_d7b50bf7b27578687ff200a936326ff_v3_5_0')\
                .validate(_payload)

        e_url = ('/v1/upgrade-patch/patch-install/pre-checks')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_d7b50bf7b27578687ff200a936326ff_v3_5_0', _api_response)

    def get_patch_precheck_status(self,
                                  pre_check_report_id=None,
                                  precheck_name=None,
                                  headers=None,
                                  **query_parameters):
        """Get the latest precheck report. User can get status of an
        individual check by passing check's name. .

        Args:
            pre_check_report_id(str): preCheckReportID query
                parameter. Precheck report's id.
            precheck_name(str): precheckName query parameter.
                Precheck name.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(pre_check_report_id, str)
        check_type(precheck_name, str)

        _params = {
            'preCheckReportID':
                pre_check_report_id,
            'precheckName':
                precheck_name,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/upgrade-patch/patch-install/pre-checks-status')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ab6c86344ca598c80d41e7b0d329248_v3_5_0', _api_response)

    def install_patch(self,
                      headers=None,
                      **query_parameters):
        """Trigger patch installation on the Cisco ISE nodes.    In Full
        patch installation, patch will be installed on PPAN
        first, and then on following nodes.    In Split patch
        installation, patch will be installed on PPAN first, and
        then on the nodes in batches of the remaining
        iterations.     A task ID is returned which can be used
        to monitor the progress of the patch installation
        process. As the patch installation triggers the Cisco
        ISE to restart, the task API becomes unavailable for a
        certain period of time. When installation is going on
        PPAN, then monitor the progress on SPAN GUI. .

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/upgrade-patch/patch-install')
        endpoint_full_url = apply_path_params(e_url, path_params)

        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d654ff67c54be93a6b720f73dc1dc_v3_5_0', _api_response)

    def patch_install_status(self,
                             pre_check_report_id=None,
                             headers=None,
                             **query_parameters):
        """Get the status of patch installation for the requested nodes.
        Precheck report Id obtained by running the precheck API
        can be passed to get the status. .

        Args:
            pre_check_report_id(str): preCheckReportID query
                parameter. Precheck report's id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(pre_check_report_id, str)

        _params = {
            'preCheckReportID':
                pre_check_report_id,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/upgrade-patch/patch-install/get-status')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b57abb188e75202ad3d10caa7cdf6a8_v3_5_0', _api_response)

    def get_installed_patches(self,
                              headers=None,
                              **query_parameters):
        """List all the installed patches in the system with the date of
        installation. .

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/upgrade-patch/patch-install/list-patch')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_aa7722a68b0b5f838831b3f9ea174bbe_v3_5_0', _api_response)

    def patch_rollback_info(self,
                            headers=None,
                            **query_parameters):
        """Get current and previous patch installed information.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/rollback/patch-rollback')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b662d1fe97b65f9a994c8d20388d403f_v3_5_0', _api_response)

    def rollback_patch(self,
                       pre_check_report_id=None,
                       headers=None,
                       **query_parameters):
        """Trigger patch rollback on the Cisco ISE nodes. Patch will be
        rolled back in other nodes first followed by P-PAN.
        Precheck report Id is mandatory which has been obtained
        by running patch rollback pre-checks API. Patch rollback
        progress can be monitored using patch rollback get
        status API.  When rollback is going on PPAN, then
        monitor the progress on SPAN GUI. .

        Args:
            pre_check_report_id(str): preCheckReportID query
                parameter. Precheck report's id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(pre_check_report_id, str)

        _params = {
            'preCheckReportID':
                pre_check_report_id,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/rollback/patch-rollback')
        endpoint_full_url = apply_path_params(e_url, path_params)

        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fc25cd5d4d8662f06ae821d213_v3_5_0', _api_response)

    def run_rollback_pre_checks(self,
                                pre_check_report_id=None,
                                pre_checks=None,
                                re_trigger=None,
                                upgrade_type=None,
                                headers=None,
                                payload=None,
                                active_validation=True,
                                **query_parameters):
        """Initiates prechecks execution on PPAN for complete deployment.
        It returns a precheck report id, which can be used to
        trigger patch rollback later. Patch rollback can happen
        in two modes :    Full Patch Rollback -> Hostname needs
        to be an empty array. First patch will get installed on
        PPAN and then it will get installed on all other nodes
        parallelly.   Sequential Patch Rollback -> Hostname
        needs to be an empty array. First patch will get
        installed on PPAN and then it will get installed on all
        other nodes sequentially.    upgradeType param can be
        specified as PATCH_ROLLBACK for full patch rollback,
        PATCH_ROLLBACK_SEQUENTIAL for sequential patch rollback.
        upgrade type is mandatory parameter. .

        Args:
            pre_check_report_id(string): preCheckReportID, property
                of the request body.
            pre_checks(list): Array of prechecks that needs to be
                executed., property of the request body
                (list of strings).
            re_trigger(boolean): reTrigger, property of the request
                body.
            upgrade_type(string): upgradeType, property of the
                request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'upgradeType':
                    upgrade_type,
                'preChecks':
                    pre_checks,
                'preCheckReportID':
                    pre_check_report_id,
                'reTrigger':
                    re_trigger,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_bd5f6c29764549995d7f64a8aca4f43_v3_5_0')\
                .validate(_payload)

        e_url = ('/v1/rollback/patch-rollback/pre-checks')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_bd5f6c29764549995d7f64a8aca4f43_v3_5_0', _api_response)

    def get_rollback_precheck_status(self,
                                     pre_check_id=None,
                                     pre_check_report_id=None,
                                     headers=None,
                                     **query_parameters):
        """Get the latest precheck report. .

        Args:
            pre_check_report_id(str): preCheckReportID query
                parameter. Precheck report's id.
            pre_check_id(str): preCheckID query parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(pre_check_report_id, str)
        check_type(pre_check_id, str)

        _params = {
            'preCheckReportID':
                pre_check_report_id,
            'preCheckID':
                pre_check_id,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/rollback/patch-rollback/pre-checks-status')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_e99695845f5491ebff92ac67af49_v3_5_0', _api_response)

    def patch_rollback_summary(self,
                               headers=None,
                               **query_parameters):
        """get the summary of patch rollback .

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/rollback/patch-rollback/summary')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ee0526c3e57252a19342978b5fd3477b_v3_5_0', _api_response)

    def patch_rollback_proceed_status(self,
                                      pre_check_report_id=None,
                                      headers=None,
                                      **query_parameters):
        """Get the status of patch rollback for the requested nodes.
        Precheck report Id obtained by running the precheck API
        is mandatory to get the status. .

        Args:
            pre_check_report_id(str): preCheckReportID query
                parameter. Precheck report's id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(pre_check_report_id, str)

        _params = {
            'preCheckReportID':
                pre_check_report_id,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/rollback/patch-rollback/get-status')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f4aba05786145944866f064786b4c9c5_v3_5_0', _api_response)

    def patch_summary(self,
                      headers=None,
                      **query_parameters):
        """get the summary of patch install process .

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/v1/upgrade-patch/patch-install/get-summary')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_bb836f58f1d153c1837f25f9a660600c_v3_5_0', _api_response)
